#!/usr/local/bin/tclsh

# MakeSurf.tcl: Menu-Driven Surface Based Processing
#
# Written by:
# Timothy M. Ellmore, LBC/NIMH/NIH, Started October 1998
#
# With lots of help, suggestions and encouragement from:
# Michael S. Beauchamp, Ph.D, LBC/NIMH/NIH
# John D. Van Horn, Ph.D, LBC/NIMH/NIH
#
# MakeSurf.tcl Implements and further automates the surface-based 
# analysis programs designed and written by Anders Dale, Martin 
# Sereno and Bruce Fischl at Massachusetts General Hospital. 
# This scheme is described in more detail in:
#
# Dale AM, Fischl B, Sereno MI. Cortical Surface-Based Analysis I:
# Segmentation and Surface Reconstruction. (to be published)
# 
# Fischl B, Sereno MI, Dale AM. Cortical Surface-Based Analysis II:
# Surface-Based Coordinate System. (to be published)
#
# MakeSurf.tcl was last modified on: March 4, 1999

#
# Beginning of Menu-Driven Surface Processing Script
#


# platform dependent definitions
global TCLPLATFORM
global TCLPLATFORMOS

# variables related to paths and environment
global VERBOSE_FLAG
global BINARIES
global BINARIES2
global MNIBINARIES
global MODEL
global MRI_DIR
global AFNIBIN
global env

# variables related to processing hierarchy
global SUBJECTS_DIR
global subjname

# variables related to trishrink and wmfilter
global FZERO 
global FMAX
global DFRAC
global ISTILT
global WHITE_HILIM
global WHITE_LOLIM
global GRAY_HILIM

set VERBOSE_FLAG 1
set EXIT_FLAG 1

# get platform dependent definitions
set TCLPLATFORM $tcl_platform(platform)
set TCLPLATFORMOS $tcl_platform(os)

if [catch { source /hippo/apps/flat/MakeSurfTools.tcl } result] {
    puts stderr $result
    exit 1
}


while {$EXIT_FLAG} {

    set MenuOption [SurfaceMenu]

    # The variable list MenuOption stores the integer or integers that
    # the user has chosen from the menu. The list is sorted in ascending
    # order so if the user has chosen a 1 then it will occupy index 0
    # in the list. A check is made below to see what integer is in index 0.
    # If the first index of the MenuOption list is not set to 1
    # then check to see if SUBJECTS_DIR and $subject name are
    # set; if not then set them so that the rest of the processes
    # know where to retrieve and store data. 

    if {(([info exists SUBJECTS_DIR] == 0 || [info exists subjname] == 0) && \
        [lindex $MenuOption 0] != 1 && [lindex $MenuOption 0] != 22)} {

	 GetSurfaceHierarchy 
    }

    
    for {set i 0} {$i < [llength $MenuOption]} {incr i} {

	set CurrentOption [lindex $MenuOption $i]

	switch -exact -- $CurrentOption {
	
	1 {
	    # Create Directory Hierarchy and Pre-Process Raw Anatomical Data		
	    if [catch { PreProcessSurf } result] {
		puts stderr $result
		exit 1
	    }
	}

	

	2 {
	    # Compute Talairach Transformation Matrix with MNI Algorithms
	    if [catch { MNINormalizeAnatomy } result] {
		puts stderr $result
		exit 1
	    }
	}

	3 {
	    # Perform 3D Bias Correction
	    if [catch { CorrectBiasField } result] {
		puts stderr $result
		exit 1
	    }
	}

	4 {
	
	    # Perform Automated Skull Removal
	    # Args: FZERO,FMAX,DFRAC,ISTILT,trishrinkmode 
	    puts stdout "Using FZERO = $env(FZERO), FMAX = $env(FMAX), DFRAC = $env(DFRAC), ISTILT = $env(ISTILT)" 
	    if [catch { StripSkull $env(FZERO) $env(FMAX) $env(DFRAC) $env(ISTILT) 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	5 {
	
	    # Classify White Matter 
	    # Args: WHITE_HILIM,WHITE_LOLIM,GRAY_HILIM
	    puts stdout "Using WHITE_HILIM = $env(WHITE_HILIM), WHITE_LOLIM = $env(WHITE_LOLIM), GRAY_HILIM = $env(GRAY_HILIM)" 
	    if [catch { ClassifyWM $env(WHITE_HILIM) $env(WHITE_LOLIM) $env(GRAY_HILIM) } result] {
		puts stderr $result
		exit 1
	    }

	}

	6 {
	
	    # Establish Cutting Planes and Fill Hemispheres
	    if [catch { FillBrain } result] {
		puts stderr $result
		exit 1
	    }

	}

	7 {
	
	    # Reconstruct the Right Hemisphere Cortical Surface
	    if [catch { ReconstructSurface 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	8 {
	
	    # Reconstruct the Left Hemisphere Cortical Surface
	    if [catch { ReconstructSurface 2 } result] {
		puts stderr $result
		exit 1
	    }

	}
	
	9 {
	
	    # Inflate the Right Hemisphere Surface Reconstruction 
	    if [catch { InflateSurface 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	10 {
	
	    # Inflate the Left Hemisphere Surface Reconstruction 
	    if [catch { InflateSurface 2 } result] {
		puts stderr $result
		exit 1
	    }

	}

	11 {
	
	    # Edit the Right Hemisphere Surface Reconstruction 
	    if [catch { EditSurface 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	12 {
	
	    # Edit the Left Hemisphere Surface Reconstruction 
	    if [catch { EditSurface 2 } result] {
		puts stderr $result
		exit 1
	    }

	}
	

	13 {

	    # Reconstruct final surfaces
	    if [catch { MakeFinalSurfaces 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	14 {

	    # Reconstruct final surfaces
	    if [catch { MakeFinalSurfaces 2 } result] {
		puts stderr $result
		exit 1
	    }

	}

        15 {

	    # Flatten the Right Hemisphere Surface Reconstruction
	    if [catch { FlattenSurfaces 1 } result] {
		puts stderr $result
		exit 1
	    }

	}

	16 {

	    # Flatten the Left Hemisphere Surface Reconstruction
	    if [catch { FlattenSurfaces 2 } result] {
		puts stderr $result
		exit 1
	    }

	}

	17 {

	    # use register to align EPI to high-resolution anatomy   
	    if [catch { DoRegister } result] {
		puts stderr $result
		exit 1
	    }

	}

	18 {

	    # paint a surface reconstruction 
	    if [catch { PaintSurface } result] {
		puts stderr $result
		exit 1
	    }
	
	}

	19 {

	    # display a painted surface reconstruction 
	    if [catch { DisplaySurface } result] {
		puts stderr $result
		exit 1
	    }

	}

        20 {

	    # launch the utilities menu
	    puts stdout "where are the utilities!"

	}

	21 {

	    # help pages
	    puts stdout "can't help you yet!"

	}

	22 {
	set EXIT_FLAG 0
	puts stdout ""
	puts stdout "Exiting Surface Analysis Script . . . ."
        puts stdout ""	
	exit 1
	}
	
	# endswitch
	}

    # endfor
    }

# endwhile
}


	

	

	

	

	

	

	





































