% dtintergroup
%
% calculate tensor-based intergroup p-value maps using permutation test
%
% input: instems1, instems2, type, outdir
%
% output: p

function dtintergroup(instems1, instems2, type, nbins, minerr, outdir)
version = '$Id: dtintergroup.m,v 1.2 2003/07/08 04:11:37 dtuch Exp $';

fprintf('dtintergroup\n');

minerr = 10^minerr;

instems1 = tokenizestring(instems1);
instems2 = tokenizestring(instems2);

n1 = size(instems1, 2);
n2 = size(instems1, 2);
n = n1 + n2;
fprintf('\n');

fprintf('size group 1: %d\n', n1);
fprintf('size group 2: %d\n', n2);

fprintf('Determining permutations for group 1. This may take a moment ... ');
tic;
c1 = nchoosek(1:n,n1);
c2 = zeros(size(c1,1), n2);
fprintf('\n');
permutations = size(c1,1);

fprintf('permutations: %d\n', permutations);
fprintf('done (%g)\n',toc);

fprintf('Determining permutations for group 2 ... ');
fprintf('\n');
tic;
for i = 1:size(c2,1)
  c2(i,:) = setdiff(1:n, c1(i,:));
  if ( (i~=1) & (mod(i-1,10^4) == 0) )
    fprintf('%d ', i-1);
  end
  if (mod(i-1,10^5) == 0)
    fprintf('\n');
  end 
end
fprintf('\n');
fprintf('done (%g)\n',toc);

fprintf('Generating random permutations look-up table ... ');
tic;
rand('state', 0');
permlook = randperm(permutations)';
p1 = find(permlook==1);
permlook(p1) = permlook(1);
permlook(1) = 1;
fprintf('done (%g)\n',toc);


[rows cols frames slice0 slices endian bext] = fmri_bfiledim(char(instems(1)));
if(isempty(rows))
  error(sprintf('ERROR: loading %s\n', char(instems(1))));
  exit;
end


fprintf('Allocating memory for group 1 ... \n');
tic;
tensor1 = repmat(int16(0), [n1, slices, rows, cols, 6]);
fprintf('done (%g)\n',toc);

fprintf('Allocating memory for group 2 ... \n');
tic;
tensor2 = repmat(int16(0), [n2, slices, rows, cols, 6]);
fprintf('done (%g)\n',toc);

% load data
fprintf('Loading group 1 data ... \n');

for member = 1:n1
  fname = char(instems1(member));
  a = fmri_ldbvolume(fname);
  if (isempty(a))
    fprintf('ERROR: loading %s\n', fname);
    exit;
  end

  n = 1;
  for i = 1:3
    for j = 1:i
      tensor1(member,:,:,:,n) = int16(128^2*a(:,:,:,i).*a(:,:,:,j));
      n = n + 1;
    end
  end

  fprintf('%d ', member-1);
end
fprintf('\n');

fprintf('Loading group 2 data ... \n');
for member = 1:n2
  fname = char(instems2(member));
  a = fmri_ldbvolume(fname);
  if (isempty(a))
    fprintf('ERROR: loading %s\n', fname);
    exit;
  end

  n = 1;
  for i = 1:3
    for j = 1:i
      tensor2(member,:,:,:,n) = int16(128^2*a(:,:,:,i).*a(:,:,:,j));
      n = n + 1;
    end
  end

  fprintf('%d ', member-1);  
end
fprintf('\n');

clear a;

fprintf('Consolidating workspace memory ... ');
tic;
pack;
fprintf('done (%g)\n',toc);


anglebins = linspace(0, pi/2, nbins);


mask = ones(slices, rows, cols);
fprintf('Calculating mask ... \n');
tic;
for slice = 1:slices
  for row = 1:rows
    for col = 1:cols
      mask(slice,row,col) = mask(slice,row,col)*
      ( (prod(prod(squeeze(double(tensor1(:,slice,row,col,:))),2),1).*
      prod(prod(squeeze(double(tensor2(:,slice,row,col,:))),2),1))>0 );
      
    end
  end
end
fprintf('done (%g)\n',toc);


fprintf('Computing p-value maps ...\n');
for slice = 1:slices
  %  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      if ( mask(slice,row,col) )

	perm = 1;
	
	err = 10^6;
	pprev = 0;
	while ( ~(perm-1) | (err > minerr) )
	  
	  T1 = 128^-2*double(tensor1(c1(permlook(perm),:),slice,row,col,:));
	  T2 = 128^-2*double(tensor2(c2(permlook(perm),:),slice,row,col,:));

	  T1 = mean(T1,1);
	  T2 = mean(T2,1);
	    
	  [V1 null] = eig(T1([1 2 3; 2 4 5; 3 5 6]));
	  [V2 null] = eig(T2([1 2 3; 2 4 5; 3 5 6]));

	  th = acos(V1(:,3)'*V2(:,3));

	  if (th>pi/2); th = pi - th; end

	  f(perm) = th;

	  [h null] = hist(f, anglebins);
	  h = h/sum(h);

	  pprev = p;
	  p = interp1q(anglebins',h',f(1));

	  err = (pprev-p)^2;

	  fprintf('s:%d r:%d c:%d n:%d p:%d/%d err:%f minerr:%f\n', ...
		  slice, row, col, perm, permutations, p, ...
		  log10(err), log10(minerr));

	  perm = perm + 1;
	end
      else
	p = 0;
      end
      pmap(slice, row, col) = p;	
    end
  end      
end

fprintf('Saving p\n');
pstem = [outdir '/p'];
fmri_svbvolume(pmap, pstem);

fprintf('dtintergroup done\n');
exit;

