%
% groupstats2
%
% Two-way group statistics
%
% input: mode, tail, instem1file, instem2file, outdir
%
% output: p
%
% mode can be: anova, ttest, ks, kw, ranksum

function groupstats2(mode, tail, instem1file, instem2file, outdir)
version = '$Id: groupstats2.m,v 1.6 2003/08/18 01:54:33 dtuch Exp $';

fprintf('groupstats2\n');
fprintf('version: %s\n\n', version);

instems1 = textread(instem1file, '%q');
instems2 = textread(instem2file, '%q');

n1 = size(instems1, 1);
n2 = size(instems2, 1);

switch (mode)
 case 'anova'
  testname = 'anova';
 
 case 'ttest'
  testname = 't-test';
  
 case 'ks'
  testname = 'Kolmogorov-Smirnov';
 
 case 'kw'
  testname = 'Kruskal-Wallis';
 
 case 'rs'
  testname = 'Wilcoxon rank sum';
end

% add tail name
if ( strcmp(mode, 'ttest') | strcmp(mode, 'ks') )
  testname = sprintf('%s (%s-sided)', testname, tail);      
end

% use sign convention for ks tail
if ( strcmp(mode, 'ttest') | strcmp(mode, 'ks') )
  switch (tail)
   
   case 'both'
    tail = 0;
   
   case 'left'
    tail = -1;
   
   case 'right'
    tail = 1;  
  
  end
end
  

fprintf('Test: %s\n', testname);

% load image dimensions
fname = char(instems1(1));
try
  [rows cols frames slice0 slices endian bext] = ...
      fmri_bfiledim(fname);
catch
  fprintf('ERROR: loading %s\n',fname);
  exit;
end

% memory
tic;
fprintf('Allocating memory ...');
data1 = zeros(n1, slices, rows, cols);
data2 = zeros(n2, slices, rows, cols);
p = zeros(slices,rows,cols);
fprintf('done (%g)\n',toc);

% load data
tic;
fprintf('Loading group 1 data (n1=%d) \n', n1);
for member = 1:n1
  fname = char(instems1(member));
  try
    fprintf('  %s\n', fname');
    a = fmri_ldbvolume(fname);
  catch
    fprintf('ERROR: loading %s\n',fname);
    exit;
  end
  data1(member,:,:,:) = a;
end
fprintf('done (%g)\n',toc);

tic;
fprintf('Loading group 2 data (n2=%d) \n', n2);
for member = 1:n2
  fname = char(instems2(member));
  try
    fprintf('  %s\n', fname');
    a = fmri_ldbvolume(fname);
  catch
    fprintf('ERROR: loading %s\n',fname);
    exit;
  end
  data2(member,:,:,:) = a;
end
fprintf('done (%g)\n',toc);

mask = squeeze(all(data1,1)).*squeeze(all(data2,1));

% generate groupnames
if (strcmp(mode, 'anova') | strcmp(mode, 'kw'))
  groupnames = [repmat({'g1'}, [1 n1]) repmat({'g2'}, [1 n2])];
end

% generate p-value map
fprintf('Computing p-value map ... \n');
for slice = 1:slices
  fprintf('%g ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      if mask(slice,row,col)

	x = squeeze(data1(:,slice,row,col));
	y = squeeze(data2(:,slice,row,col));	

	sgn = sign(mean(x)-mean(y));

	switch (mode)	 
	  
	 case 'anova'
	  pvalue = anova1([x y], groupnames);
	  
	 case 'ttest'
	  [h pvalue ci] = ttest2(x, y, .05, tail);
	  
	 case 'kw'
	  pvalue = kruskalwallis([x y], groupnames);
	  
	 case 'ks'
	  pvalue = kstest(X,cdf,.05,tail);
	  
	 case 'ranksum'
	  [pvalue h] = ranksum(x,y);
	end

	if (pvalue)
	  p(slice, row, col) = -sgn*log10(pvalue);
	end

      end      
    end      
  end
end

fprintf('Saving p\n');
outstem = [outdir '/p'];
fmri_svbvolume(p, outstem);
clear p;

mean1 = squeeze(mean(data1,1));
fprintf('Saving mean001\n');
outstem = [outdir '/mean001'];
fmri_svbvolume(mean1, outstem);
clear mean1;

mean2 = squeeze(mean(data2,1));
fprintf('Saving mean002\n');
outstem = [outdir '/mean002'];
fmri_svbvolume(mean2, outstem);
clear mean2;

std1 = squeeze(std(data1,0,1));
fprintf('Saving std001\n');
outstem = [outdir '/std001'];
fmri_svbvolume(std1, outstem);
clear std1;

std2 = squeeze(std(data2,0,1));
fprintf('Saving std002\n');
outstem = [outdir '/std002'];
fmri_svbvolume(std2, outstem);
clear std2;


fprintf('groupstats2 done\n');
exit;






