% fast_dsrecon
%
% input:
% instem, gradient_filename, fastestdim, width, outdir
%
% optional input:
% maskbg
%
% output:
% pdf, odf, rto, lowb, avg,

function dsrecon(instem, gradientfilename, smootwidth, odfsigma, ...
		 fastestdim, maskbg, interp, outdir)

version = '$Id: dsrecon.m,v 1.2 2003/07/08 04:11:38 dtuch Exp $';

fprintf('dsrecon\n');

% check args
if strcmp(maskbg, 'null')
  clear maskbg;
else
  maskbg = str2num(maskbg);
end

if (exist('maskbg') & (abs(maskbg) > 1 ) )
  fprintf('ERROR: maskbg parameter m (= %d) must be between -1 and 1\n', ...
	  maskbg);
  exit;
end

% hard-wired
recondirfilename = ([diffusiontoolbox '/geometry/dodeca5-755dir.txt']);
interpmethod = ['*' interp];

[rows cols frames slice0 slices endian bext] = fmri_bfiledim(instem);
if(isempty(rows))
  fprintf('ERROR: loading %s\n',instem);
  return;
end

gradients = load(gradientfilename);
[dummy, lowbindex] = min(diag(gradients*gradients'));

recondirs = load(recondirfilename);
nrecondirs = size(recondirs, 1);

directions = length(gradients);
averages = frames/directions;

tic;
fprintf('Computing kernels ... ');
scale = 0;
while (scale < 10) & ...
      (size(unique(round(scale*gradients),'rows'),1) ~= directions)
  scale = scale + 1;
end

gradients = round(scale*gradients);

hfov = max(abs(gradients(:)));
fov = 2*hfov + 1; % width of q-lattice 
center = (fov+1)/2;

steps = round(4.1*fov); % # of steps in RP
samples = linspace(-hfov,hfov,steps);


%kernel = RadialProjectionKernel(fov, steps, 0, nrecondirs);
points = RadialProjectionSamplePoints(samples, recondirs);
fprintf('done (%g)\n',toc);

tic;
fprintf('Loading data ... ');
rawdata = fmri_ldbvolume(instem);
if(isempty(rawdata))
  fprintf('ERROR: loading %s\n',instem);
  return;
end
fprintf('done (%g)\n',toc);

dims_avgs_fastest = [slices rows cols averages directions];
dims_dirs_fastest = [slices rows cols directions averages];

if strcmp(fastestdim, 'auto')
  fprintf('Autodetecting fastest dimension ... ');
  fastestdim = fastestdimdetect(rawdata, slices, rows, cols, averages, ...
				directions);
  fprintf('done (%g)\n',toc);
end

if strcmp(fastestdim, 'averages');
  fprintf('Fastest dimension: averages\n');
  dims = dims_avgs_fastest;
  avgindex = 4;
else
  fprintf('Fastest dimension: directions\n');
  dims = dims_dirs_fastest;
  avgindex = 5;
end

fprintf('Reshaping ... ');
rawdata = reshape(rawdata, dims);
fprintf('done (%g)\n',toc);

fprintf('Averaging raw data ... ');
avgrawdata = squeeze(mean(rawdata, avgindex));
globalavgdata = sum(avgrawdata,4);
fprintf('done (%g)\n',toc);
clear rawdata;

fprintf('Image dimensions:\n');
fprintf(' slices      %d\n', slices);
fprintf(' rows        %d\n', rows);
fprintf(' cols        %d\n', cols);
fprintf(' directions  %d\n', directions);
fprintf(' averages    %d\n', averages);

if exist('maskbg')
  tic;
  fprintf('Calculating background threshold value ... \n');
  smoothedglobal = convn(globalavgdata, gausskernel(3, [1 1 1], ...
						    1),'same');
  valley = findvalley(smoothedglobal, ...
                      prod(size(smoothedglobal))^(2/3));

  adjustedvalley = calculatethresh(smoothedglobal, valley, maskbg);
  mask = smoothedglobal>adjustedvalley;

  bgfrac = sum(smoothedglobal(:)<valley)/prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (unadjusted)\n', 100* ...
	  bgfrac);

  bgfrac = sum(smoothedglobal(:)<adjustedvalley)/ ...
	   prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (m adjusted)\n', 100* ...
	  bgfrac);
  fprintf('  done (%g)\n',toc);

  clear smoothedglobal;

else

  mask = ones(size(globalavgdata));

end

tic;
fprintf('Allocating memory...');
odfmap = zeros(slices, rows, cols, nrecondirs);
pdfmap = zeros(slices, rows, cols, fov^3);
rtomap = zeros(slices, rows, cols);
gfamap = zeros(slices, rows, cols);
fprintf(' done (%g)\n',toc);

tic;
fprintf('Consolidating workspace memory ... ');
try
  pack;
catch
  fprintf('\nWARNING: unable to consolidate memory ... ');
end
fprintf('done (%g)\n',toc);

q = gradients + hfov + 1;

smoothwidthflag = exist('smoothwidth');

fprintf('Computing PDFs and ODFs...\n');
for slice = 1:slices
  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      zeroflag = 0;

      data = squeeze(avgrawdata(slice,row,col,:));      
      if ( ~mask(slice,row,col) | ...
	   any(isinf(data)) | ...
	   any(isnan(data)) )
	zeroflag = 1;
      end
      
      if (~zeroflag)
	E = zeros(fov,fov,fov);
	for i = 1:directions
	  E(q(i,1), q(i,2), q(i,3)) = data(i);
	end
	
	pdftemp = fftshift(abs(fft(E)),1);
	
	if smoothwidthflag & smoothwidth
	  pdftemp = smooth3(pdftemp, 'gaussian', smoothwidth*[1 1 1]); 
	end
	
	pdftemp = pdftemp/sum(sum(sum(pdftemp)));
	rtotemp = interp3(pdftemp, center, center, center);
	odftemp = pdf2odf(pdftemp, points, interpmethod, nrecondirs, ...
			  steps);
	gfatemp = gfa(odftemp);
      
      end
      
      if (zeroflag)
	pdftemp = zeros(fov^3,1);
	odftemp = zeros(nrecondirs,1);
	rtotemp = 0;
	gfatemp = 0;
      end
      
      pdfmap(slice, row, col,:) = pdftemp(:);
      odfmap(slice, row, col,:) = odftemp;
      rtomap(slice, row, col) = rtotemp;
      gfamap(slice, row, col) = gfatemp;
      
    end
  end  
end

fprintf('Saving avg\n');
avgstem = sprintf('%s/avg',outdir);
fmri_svbvolume(avgrawdata, avgstem);

fprintf('Saving rto\n');
rtostem = sprintf('%s/rto',outdir);
fmri_svbvolume(rtomap, rtostem);

fprintf('Saving gfa\n');
gfastem = sprintf('%s/gfa',outdir);
fmri_svbvolume(gfamap, gfastem);

fprintf('Saving pdf\n');
pdfstem = sprintf('%s/pdf',outdir);
fmri_svbvolume(pdfmap, pdfstem);

fprintf('Saving odf\n');
odfstem = sprintf('%s/odf',outdir);
fmri_svbvolume(odfmap, odfstem);

fprintf('dsrecon done\n');

exit

