% qballrecon
%
% reconstruct ODF maps from raw HAR data
%
% input: instem, gradientfilename, fastestdim, outdir
%
% optional input: odfsigma
%
% output: odf, avg, diravg, gfa

function qballrecon(instem, gradientfilename, odfsigma, fastestdim, maskbg, outdir)
version = '$Id: qballrecon.m,v 1.3 2003/07/08 17:48:00 dtuch Exp $';

fprintf('qballrecon\n');

% check args
if strcmp(maskbg, 'null')
  clear maskbg;
else
  maskbg = str2num(maskbg);
end
    
if (exist('maskbg') & (abs(maskbg) > 1 ) )
  fprintf('ERROR: maskbg parameter m (= %d) must be between -1 and 1\n', ...
	  maskbg);
  exit;
end


% hard-wired
recondirfilename = 'dodeca5-755dir.txt';
		    
rbfsigma = 7.5;
datasigma = odfsigma;

try
  [rows cols frames slice0 slices endian bext] = fmri_bfiledim(instem);
catch
  fprintf('ERROR: loading %s\n',instem);
  exit;
end

gradients = load(gradientfilename);
gradients = gradients/max(sqrt(diag(gradients*gradients')));
[dummy, lowbindex] = min(diag(gradients*gradients'));

datasigma = datasigma*pi/180;
odfsigma = odfsigma*pi/180;
rbfsigma = rbfsigma*pi/180;

recondirs = load(recondirfilename);
recondirs = recondirs/max(sqrt(diag(recondirs*recondirs')));
nrecondirs = size(recondirs,1);

directions = length(gradients);
averages = frames/directions;

tic;
fprintf('Loading data ... ');
try
  rawdata = fmri_ldbvolume(instem);
catch
  fprintf('ERROR: loading %s\n',instem);
  exit  
end
fprintf('done (%g)\n',toc);

dims_avgs_fastest = [slices rows cols averages directions];
dims_dirs_fastest = [slices rows cols directions averages];

if strcmp(fastestdim, 'auto')
  fprintf('Autodetecting fastest dimension ... ');
  fastestdim = fastestdimdetect(rawdata, slices, rows, cols, averages, directions);
  fprintf('done (%g)\n',toc);
end

switch (fastestdim)
 case 'averages'
  fprintf('Fastest dimension: averages\n');
  dims = dims_avgs_fastest;
  avgindex = 4;
 otherwise
  fprintf('Fastest dimension: directions\n');
  dims = dims_dirs_fastest;
  avgindex = 5;
end

fprintf('Reshaping ... ');
rawdata = reshape(rawdata, dims);
fprintf('done (%g)\n',toc);

fprintf('Averaging raw data ... ');
avgrawdata = squeeze(mean(rawdata, avgindex));
globalavgdata = sum(avgrawdata,4);
fprintf('done (%g)\n',toc);
clear rawdata;

%tic;
%fprintf('Calculating SNR ...\n');
%[unattensnr attensnr] = calcsnr(globalavgdata, -1, avgrawdata, -1);
%fprintf('  SNR (unattenuated)  %f\n', unattensnr);
%fprintf('  SNR (attenuated)    %f\n', attensnr);
%fprintf('  done (%g)\n',toc);

fprintf('Image dimensions:\n');
fprintf('  slices      %d\n', slices);
fprintf('  rows        %d\n', rows);
fprintf('  cols        %d\n', cols);
fprintf('  directions  %d\n', directions);
fprintf('  averages    %d\n', averages);
 

fprintf('Computing kernels ... ');
tic;
circlesteps = 48;
greatcirclepts = GreatCircleSamplePoints(recondirs, circlesteps);
weightskernel = SphericalRBFWeightsKernel(gradients, recondirs, rbfsigma, 10^-10);
odfkernel = SphericalRBFODFKernel(greatcirclepts, recondirs, rbfsigma);
smoothdatakernel = SmoothDataKernel(gradients, datasigma);
smoothodfkernel = SmoothODFKernel(recondirs, odfsigma);
totalkernel = odfkernel*weightskernel*smoothdatakernel;
totalkernel = smoothodfkernel*totalkernel;
%clear greatcirclepts weightskernel odfkernel smoothdatakernel smoothodfkernel;

fprintf('done (%g)\n',toc);

if exist('maskbg')
  tic;
  fprintf('Generating mask ... \n');
  smoothedglobal = convn(globalavgdata, gausskernel(3, [1 1 1], 1),'same');
  valley = findvalley(smoothedglobal, ...
		      prod(size(smoothedglobal))^(2/3));
  
  adjustedvalley = calculatethresh(smoothedglobal, valley, maskbg);
  mask = smoothedglobal>adjustedvalley;
  
  bgfrac = sum(smoothedglobal(:)<valley)/prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (unadjusted)\n', 100*bgfrac);

  bgfrac = sum(smoothedglobal(:)<adjustedvalley)/prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (m adjusted)\n', 100*bgfrac);
  fprintf('  done (%g)\n',toc);
  
  clear smoothedglobal;
  
else
  mask = int8(ones(size(globalavgdata)));
end

fprintf('Masking out Infs and NaNs ... \n');
tic;
mask = mask.*~isinf(globalavgdata).*~isnan(globalavgdata);
fprintf('  done (%g)\n',toc);

tic;
fprintf('Consolidating workspace memory ... ');
try
  pack;
catch
  fprintf('\nWARNING: unable to consolidate memory ... ');
end
fprintf('done (%g)\n',toc);

tic;
fprintf('Allocating memory ... ');
odfmap = zeros(slices, rows, cols, nrecondirs);
gfamap = zeros(slices, rows, cols);
fprintf('done (%g)\n',toc);

fprintf('Computing ODFs ... \n');
for slice = 1:slices
  tic;
  fprintf('processing slice %d/%d ', slice, slices);
  for row = 1:rows
    for col = 1:cols
      zeroflag = 0;
      
      data = squeeze(avgrawdata(slice,row,col,:));      
      if ( ~mask(slice,row,col ) )	
	zeroflag = 1;
      end
      
      if (~zeroflag)
	
	odftemp = totalkernel*data;
	odftemp = odftemp / sum(odftemp);
	gfatemp = gfa(odftemp);
	
      end
      
      if (zeroflag)
	odftemp = zeros(nrecondirs,1);
	gfatemp = 0;
      end
      
      odfmap(slice,row,col,:) = odftemp;
      gfamap(slice,row,col) = gfatemp;
    end
  end  
  fprintf('(%g)\n', toc);
end

fprintf('Saving avg\n');
avgstem = sprintf('%s/avg',outdir);
fmri_svbvolume(avgrawdata, avgstem);

fprintf('Saving diravg\n');
diravgstem = sprintf('%s/diravg',outdir);
fmri_svbvolume(globalavgdata, diravgstem);

fprintf('Saving gfa\n');
gfastem = sprintf('%s/gfa',outdir);
fmri_svbvolume(gfamap, gfastem);

fprintf('Saving odf\n');
odfstem = sprintf('%s/odf',outdir);
fmri_svbvolume(odfmap, odfstem);

fprintf('qballrecon done\n');

exit;


