% raw2tensor
%
% reconstruct diffusion tensor and scalars from raw data
%
% input: instem, gradientfilename, fastestdim, bvalue, outdir
%
% optional input: maskbg, trmax
%
% output: lowb, avgraw, dtensor

function raw2tensor(instem, gradientfilename, bvalue, fastestdim, maskbg, noisereg, outdir)
version = '$Id: raw2tensor.m,v 1.5 2003/07/28 13:41:59 dtuch Exp $';

fprintf('raw2tensor\n');

% check args
if strcmp(maskbg, 'null')
  clear maskbg;
elseif ischar(maskbg)
  maskbg = str2num(maskbg);
end

if (exist('maskbg') & (abs(maskbg) > 1 ) )
  fprintf('ERROR: maskbg parameter m (= %d) must be between -1 and 1\n', ...
	  maskbg);
  exit;
end

try
  [rows cols frames slice0 slices endian bext] = ...
      fmri_bfiledim(instem);
catch
  fprintf('ERROR: loading %s\n',instem);
  exit;
end

gradients = load(gradientfilename);
gradients = gradients/max(sqrt(diag(gradients*gradients'))); 
[dummy, lowbindex] = min(diag(gradients*gradients'));        

directions = length(gradients);

if ~ismember(directions, factor(frames)) & (directions ~= frames)
  fprintf('ERROR: number of frames (%d) not an integer multiple of the number of directions (%d)', frames, directions);
  exit;
end

averages = frames/directions;

% calculate B-matrix
for i = 1:directions
  bvec = reshape(gradients(i,:)'*gradients(i,:), [1 9]);
  bvec = bvec([1 2 3 5 6 9]).*[1 2 2 1 2 1];
  B(i,:) = [bvalue*bvec 1];
end

% load data
tic;
fprintf('Loading data ... ');
try
  rawdata = fmri_ldbvolume(instem);
catch
  fprintf('ERROR: loading %s\n',instem);
  exit;
end
fprintf('done (%g)\n',toc);

% determine data partitioning
dims_avgs_fastest = [slices rows cols averages directions];
dims_dirs_fastest = [slices rows cols directions averages];

if strcmp(fastestdim, 'auto')
  fprintf('Autodetecting fastest dimension ... ');
  fastestdim = fastestdimdetect(rawdata, slices, rows, cols, averages, directions);
  fprintf('done (%g)\n',toc);
end

if strcmp(fastestdim, 'averages');
  fprintf('Fastest dimension: averages\n');
  dims = dims_avgs_fastest;
  avgindex = 4;
else 
  fprintf('Fastest dimension: directions\n');
  dims = dims_dirs_fastest;
  avgindex = 5;
end

fprintf('Reshaping ... ');
rawdata = reshape(rawdata, dims);
fprintf('done (%g)\n',toc);

fprintf('Averaging raw data ... ');
avgrawdata = squeeze(mean(rawdata, avgindex));
globalavgdata = sum(avgrawdata,4);
avgdwi = squeeze(mean(avgrawdata(:,:,:,setdiff(1:directions, lowbindex)),4));
lowb = avgrawdata(:, :, :, lowbindex);
fprintf('done (%g)\n',toc);
 
tic;
fprintf('Calculating SNR ...\n');
[unattensnr attensnr] = calcsnr(globalavgdata, lowb, avgrawdata, lowbindex);
fprintf('  SNR (unattenuated)  %f\n', unattensnr);
fprintf('  SNR (attenuated)    %f\n', attensnr);
fprintf('  done (%g)\n',toc);

if ( noisereg & (directions == 7) )
  fprintf(['INFO: Number of directions = 7. Noise regularization' ...
	   ' disabled. Number of directions must be > 7 for noise' ...
	   ' regularization\n']);
  noisereg = 0;
end

if (noisereg)	     
  bg = avgrawdata.*(repmat((~any(avgrawdata==0,4)).*background, [1 1 1 directions]));
  bg = bg(bg>0);
  bg = reshape(bg, [size(bg,1)/directions directions]);
  sigmainv = pinv(cov(log(bg)));
  W = pinv(B'*sigmainv*B)*B'*sigmainv;
else
  W = pinv(B);
end

fprintf('Image dimensions:\n');
fprintf('  slices      %d\n', slices);
fprintf('  rows        %d\n', rows);
fprintf('  cols        %d\n', cols);
fprintf('  directions  %d\n', directions);
fprintf('  averages    %d\n', averages);
fprintf('  lowbdir     %d\n', lowbindex);
fprintf('  B-mat cond# %f\n', cond(B));

if exist('maskbg')

  tic;
  fprintf('Calculating background threshold value ... \n');
  valley = findvalley(globalavgdata, prod(size(globalavgdata))^(2/3));
  adjustedvalley = calculatethresh(globalavgdata, valley, maskbg);
  mask = globalavgdata > adjustedvalley;

  bgfrac = sum(globalavgdata(:)<valley)/prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (unadjusted)\n', 100*bgfrac);

  bgfrac = sum(globalavgdata(:)<adjustedvalley)/prod(size(globalavgdata));
  fprintf('  background is %g%% of volume (m adjusted)\n', 100*bgfrac);
  fprintf('  done (%g)\n',toc);

else

  mask = ones(size(globalavgdata));

end

tic;
fprintf('Allocating memory ...');
D = zeros(slices, rows, cols, 3, 3);
fprintf(' done (%g)\n',toc);

fprintf('Computing tensors ...\n');
for slice = 1:slices
  tic;
  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      svec = squeeze(avgrawdata(slice,row,col,:));      
      
      if ( ~mask(slice,row,col) | ...
	   any(svec <= 0) | ...
	   any(isinf(svec)) | ...
	   any(isnan(svec)) )
	continue;
      end
      
      d = -W*log(svec); 	
      D(slice,row,col,:,:) = reshape(d([1 2 3 2 4 5 3 5 6]), [3 3]);      
      
    end
  end  
end

fprintf('\n');

lowb = lowb.*mask;

fprintf('Saving lowb\n');
outstem = sprintf('%s/lowb', outdir);
fmri_svbvolume(lowb, outstem);

fprintf('Saving dtensor\n');
outstem = sprintf('%s/dtensor', outdir);
nf = 9;
tmp = squeeze(reshape(D, [slices rows cols nf]));
fmri_svbvolume(tmp, outstem);

fprintf('raw2tensor done\n');
exit;

