%
% tensor2eig
%
% reconstruct eigensystem and related scalars from diffusion tensor file
%
% input:
% instem, outdir
%
% optional input:
% trmax
%
% output:
% fa, trace, sheet, eigval, eigvec1, eigvec2, eigvec3

function tensor2eig(instem, trmax, outdir)
version = '$Id: tensor2eig.m,v 1.4 2003/07/28 13:41:59 dtuch Exp $';
 
fprintf('tensor2eig\n');

if strcmp(trmax, 'null')
  trmaxflag = 0;
elseif ischar(trmax)
  trmax = str2num(trmax);
  trmaxflag = 1;
end

[rows cols frames slice0 slices endian bext] = fmri_bfiledim(instem);
if(isempty(rows))
  error(sprintf('ERROR: loading %s\n',instem));
  exit;
end 

% load data
tic;
fprintf('Loading data ... ');
dtensor = fmri_ldbvolume(instem);
if (isempty(dtensor))
  fprintf('ERROR: loading %s\n',instem);
  exit;
end
fprintf('done (%g)\n',toc);
dtensor = reshape(dtensor, [slices rows cols 3 3]);

sparsemode = 0;

if (sparsemode)
  tic;
  fprintf('Sparsifying data ... ');
  dtensor = sparse(dtensor(:));
  fprintf('done (%g)\n', toc);
end

fprintf('Image dimensions:\n');
fprintf('  slices/rows/cols = %d/%d/%d\n', slices, rows, cols);

tic;
fprintf('Allocating memory ...');
eigenvectors = zeros(slices, rows, cols, 3, 3);
eigenvalues = zeros(slices, rows, cols, 3);
fa    = zeros(slices, rows, cols);
trace = zeros(slices, rows, cols);
fprintf(' done (%g)\n',toc);


fprintf('Computing eigensystems ...\n');
for slice = 1:slices
  fprintf('%d ', slice - 1);
  for row = 1:rows
    for col = 1:cols

      if (sparsemode)
	d = dtensor(sub2ind([slices rows cols 3 3], ...
			    slice, row, col, 1, 1));
      else
	d = squeeze(dtensor(slice, row, col, 1, 1));
      end
      
      if (~d) continue; end;

      if (trmaxflag)
	if (sparsemode)
	  tracetmp = 0;
	  for i = 1:3
	    tracetmp = tracetmp + ...
		dtensor(sub2ind([slices rows cols 3 3], ...
				slice, row, col, i, i));
	  end
	else
	  tracetmp = ...
	      sum(diag(squeeze(dtensor(slice, row, col, :, :))));
	end
	
	if (tracetmp > trmax) continue; end;
      
      end
      
	
      if (sparsemode)
	for i = 1:3
	  for j = 1:3
	    D(i,j) = dtensor(sub2ind([slices rows cols 3 3], ...
				     slice, row, col, i, j));
	  end
	end
      else
 	D = squeeze(dtensor(slice,row,col,:,:));
      end
      
      [evec, el] = eig(D);
      el = diag(el);
      
      eigenvalues(slice,row,col,:) = el(3:-1:1);
      eigenvectors(slice,row,col,:,:) = evec(:,3:-1:1);
      fa(slice,row,col) = sqrt(3)*std(el)/norm(el);
    end
  end  
end

fprintf('\n');

fprintf('Saving fa\n');
outstem = sprintf('%s/fa', outdir);
fmri_svbvolume(fa, outstem);

clear fa;

fprintf('Saving eigval\n');
outstem = sprintf('%s/eigval', outdir);
fmri_svbvolume(eigenvalues, outstem);

fprintf('Saving trace\n');
outstem = sprintf('%s/trace', outdir);
fmri_svbvolume(squeeze(sum(eigenvalues,4)), outstem);

clear eigenvalues trace;

fprintf('Saving eigvec1\n');
outstem = sprintf('%s/eigvec1', outdir);
fmri_svbvolume(squeeze(eigenvectors(:,:,:,:,1)), outstem);

fprintf('Saving eigvec2\n');
outstem = sprintf('%s/eigvec2', outdir);
fmri_svbvolume(squeeze(eigenvectors(:,:,:,:,2)), outstem);

fprintf('Saving eigvec3\n');
outstem = sprintf('%s/eigvec3', outdir);
fmri_svbvolume(squeeze(eigenvectors(:,:,:,:,3)), outstem);

%fprintf('Saving H\n');
%Hstem = [outdir '/H'];
%tmp = squeeze(reshape(H, [slices rows cols 9]));
%fmri_svbvolume(tmp, Hstem);

fprintf('tensor2eig done\n');
exit;

