#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>

#include "pkfutil.h"

static const char *pkfind_src_version = "$Id: pkfutil.c,v 1.3 2000/01/06 21:31:24 greve Exp greve $";
static char tmpstr[10000];

/* ------------------------------------------------------ */
void *AllocCluster(int nvoxels)
{
  CLUSTER * c;

  c = (CLUSTER *) calloc(sizeof(CLUSTER),1);
  if(nvoxels > 0)
    c->ss = (VOLSS *) calloc(sizeof(VOLSS),nvoxels);
  c->nvxs = nvoxels;

  return((void *)c);
}

/* ------------------------------------------------------ */
int FreeCluster(void **ppc)
{
  CLUSTER *c;

  c = (CLUSTER *)(*ppc);
  if(c->ss) free(c->ss);
  free(c);
  *ppc = NULL;
  return(0);
}
/* ------------------------------------------------------ */
void *CopyCluster(void *dest, void *src)
{
  CLUSTER *clsrc, *cldest;

  clsrc  = (CLUSTER *)src;
  cldest = (CLUSTER *)dest;

  if(!cldest) cldest = AllocCluster(clsrc->nvxs);

  cldest->ipeak = clsrc->ipeak;
  cldest->nvxs  = clsrc->nvxs;

  if(clsrc->nvxs){
    cldest->ss = (VOLSS *) realloc(cldest->ss,clsrc->nvxs*sizeof(VOLSS));
    memcpy(cldest->ss,clsrc->ss,clsrc->nvxs*sizeof(VOLSS));
  }

  return(cldest);
}
/* ------------------------------------------------------ */
int DumpCluster(FILE *fp, CLUSTER *c, IMGVOL *iv, char const *fmt)
{
  int n;

  fprintf(fp,"nvxs  = %d\n",c->nvxs);
  fprintf(fp,"ipeak = %d\n",c->ipeak);
  for(n=0;n<c->nvxs;n++){
    fprintf(fp,"%4d   %3d %3d %3d   ",n,c->ss[n].s,c->ss[n].r,c->ss[n].c);
    if(iv) fprintf(fp,fmt,iv->vox[c->ss[n].s][c->ss[n].r][c->ss[n].c]);
    fprintf(fp,"\n");
  }
  return(0);
}
/* ------------------------------------------------------ */
int AddToCluster(VOLSS *ss, CLUSTER *cl)
{
  int n;

  n = cl->nvxs ;
  cl->nvxs++;

  cl->ss = (VOLSS *) realloc(cl->ss, sizeof(VOLSS)*cl->nvxs);
  memcpy(&(cl->ss[n]), ss, sizeof(VOLSS));

  return(0);
}
/* ------------------------------------------------------ */
void *AllocImgVol(int nslices, int nrows, int ncols)
{
  IMGVOL *iv;
  int r,s,nvox;
  float *pdata;

  iv = (IMGVOL *)calloc(sizeof(IMGVOL),1);
  iv->nrows   = nrows;
  iv->ncols   = ncols;
  iv->nslices = nslices;

  nvox = nrows*ncols*nslices;
  iv->data = (float *) calloc(sizeof(float),nvox);
  iv->dataflag = (char *) calloc(sizeof(char),nvox);
  iv->nvox = nvox;

  pdata = iv->data;
  iv->vox = (float ***) calloc(sizeof(float **),nslices);
  iv->voxflag = (char ***) calloc(sizeof(char **),nslices);
  for(s=0;s<nslices;s++){
    iv->vox[s] = (float **) calloc(sizeof(float *),nrows);
    iv->voxflag[s] = (char **) calloc(sizeof(char *),nrows);
    for(r=0;r<nrows;r++){
      iv->vox[s][r] = pdata;
      pdata += ncols;
    }
  }
  return((void *)iv);
}
/* ------------------------------------------------------ */
void *CopyImgVol(void *dest, void *src)
{
  IMGVOL *ivsrc, *ivdest;

  ivsrc  = (IMGVOL *) src;
  ivdest = (IMGVOL *) dest;

  if(!ivdest) 
    ivdest = AllocImgVol(ivsrc->nslices,ivsrc->nrows,ivsrc->ncols);
  ivdest->rowres   = ivsrc->rowres;
  ivdest->colres   = ivsrc->colres;
  ivdest->sliceres = ivsrc->sliceres;
  memcpy(ivdest->data,ivsrc->data,sizeof(float)*ivdest->nvox);

  return((void *)ivdest);
}
/* ------------------------------------------------------ */
int FreeImgVol(void **ppImgVol)
{
  int s;
  IMGVOL *iv;

  iv = (IMGVOL *) *ppImgVol;

  for(s=0;s<iv->nslices;s++){
    if(iv->vox[s]) free(iv->vox[s]);
    if(iv->voxflag[s]) free(iv->voxflag[s]);
  }
  if(iv->vox) free(iv->vox) ;
  if(iv->voxflag) free(iv->voxflag) ;

  if(iv->data) free(iv->data);
  if(iv->dataflag) free(iv->dataflag);

  free(iv);
  *ppImgVol = NULL;
  return(0);
}
/* ------------------------------------------------------ */
IMGVOL *RandImgVol(int nslices, int nrows, int ncols,
		   float mean, float std, int seed)
{
  int vx;
  IMGVOL *iv;

  srand48(seed);

  iv = AllocImgVol(nslices,nrows,ncols);

  for(vx = 0; vx < iv->nvox; vx++)
    iv->data[vx] = mean + std * (drand48()-0.5);

  return(iv);
}

/* ------------------------------------------------------ */
int DumpImgVol(FILE *fp, char const *fmt, void *ImgVol)
{
  int s, r, c, n;
  IMGVOL *iv;

  iv = (IMGVOL *) ImgVol;

  fprintf(fp,"nslices  = %3d\n",iv->nslices);
  fprintf(fp,"nrows    = %3d\n",iv->nrows);
  fprintf(fp,"ncols    = %3d\n",iv->ncols);
  fprintf(fp,"sliceres = %g\n",iv->sliceres);
  fprintf(fp,"rowres   = %g\n",iv->rowres);
  fprintf(fp,"colres   = %g\n",iv->colres);

  n = 0;
  for(s=0;s<iv->nslices;s++){
    for(r=0;r<iv->nrows;r++){
      for(c=0;c<iv->ncols;c++){
	fprintf(fp,"%6d %3d %3d %3d  %d   %g\n",
		n,s,r,c,iv->voxflag[s][r][c],iv->vox[s][r][c]);
	n++;
      }
    }
  }
  return(0);
}
/* ------------------------------------------------------ */
void * AllocClusterList(int nclusters)
{
  CLIST *clist;
  int n;

  clist = (CLIST *) calloc(sizeof(CLIST),1);
  clist->nclusters = nclusters;
  if(nclusters > 0){
    clist->cluster = (CLUSTER **) calloc(sizeof(CLUSTER *),nclusters);
    for(n=0; n < clist->nclusters; n++)
      clist->cluster[n] = AllocCluster(0);
  }
  clist->sorted = 0;
  return((void *)clist);
}
/* ------------------------------------------------------ */
void * CopyClusterListHeader(void *d, void *s)
{
  CLIST *clistdest;
  CLIST *clistsrc;

  clistdest = (CLIST *) d;
  clistsrc  = (CLIST *) s;

  if(!clistdest) clistdest = AllocClusterList(0);
  clistdest = AllocClusterList(0);
  clistdest->valthresh   = clistsrc->valthresh;
  clistdest->distthresh  = clistsrc->distthresh;
  clistdest->sizethresh  = clistsrc->sizethresh;
  clistdest->pksign      = clistsrc->pksign;
  clistdest->sorted      = clistsrc->sorted;
  clistdest->nclusters   = 0;

  return((void *)clistdest);
}



/* ------------------------------------------------------------- */
int mymax(int a, int b)
{
  if(a>b) return(a);
  return(b);
}
int mymin(int a, int b)
{
  if(a<b) return(a);
  return(b);
}

/* ------------------------------------------------------------- */
CLUSTER_LIST *GetSuperVoxels(IMGVOL *iv, float valthresh, int pksign, int nbsize)
{
  CLIST *clist;
  CLUSTER *cl;
  int add,vx, nover, s, r, c;
  int snb, rnb, cnb;
  int snb1, snb2, rnb1, rnb2, cnb1, cnb2;
  float vnb;

  cl = AllocCluster(1);
  clist = AllocClusterList(0);
  clist->valthresh = valthresh;
  clist->pksign    = pksign;
  clist->nbsize    = nbsize;

  nover = 0;
  vx = 0;
  for(s=0;s<iv->nslices;s++){
    for(r=0;r<iv->nrows;r++){
      for(c=0;c<iv->ncols;c++){

	if(iv->data[vx] != iv->vox[s][r][c]){
          printf("GetSuperVoxels(): %d\n",__LINE__);
	  printf("ERROR: vx = %d, s=%d, r=%d, c=%d, data=%g, vox = %g\n",
		 vx,s,r,c,iv->data[vx],iv->vox[s][r][c]);
	  printf("ns=%d, nr=%d, nc=%d\n",iv->nslices,iv->nrows,iv->ncols);
	  printf("nover = %d\n",nover);
	  exit(1);
	}

	if(s==25 && r==47 && c==30){
	  printf("src: v = %g\n",iv->data[vx]);
	}

	if(SuperThreshold(iv->data[vx],valthresh,pksign)){
	  add = 1;

	  if(nbsize > 0){
	    snb1 = mymax(s-nbsize,0);
	    snb2 = mymin(s+nbsize,iv->nslices-1);
	    rnb1 = mymax(r-nbsize,0);
	    rnb2 = mymin(r+nbsize,iv->nrows-1);
	    cnb1 = mymax(c-nbsize,0);
	    cnb2 = mymin(c+nbsize,iv->ncols-1);

	    for(snb=snb1; snb <= snb2; snb++){
	      for(rnb=rnb1; rnb <= rnb2; rnb++){
		for(cnb=cnb1; cnb <= cnb2; cnb++){
		  if(snb==s && rnb==r && cnb==c) continue;
		  vnb = iv->vox[snb][rnb][cnb];
		  if(!SuperThreshold(iv->data[vx],vnb,pksign)){
		    add = 0;
		  }
		} /* cnb */
	      } /* rnb */
	    } /* snb */
	  }

	  if(add){
	    cl->ss->s = s; cl->ss->r = r; cl->ss->c = c;
	    AddToClusterList(cl,clist);
	    nover ++;
	  }
	}
	vx++;
      }
    }
  }

  FreeCluster((void **)&cl);
  return(clist);
}
#if 0
/* ------------------------------------------------------------- */
CLUSTER_LIST *GetSuperVoxels(IMGVOL *iv, float valthresh, int pksign)
{
  CLIST *clist;
  int vx, nover, s, r, c;
  VOLSS ss;

  nover = 0;
  for(vx = 0; vx < iv->nvox; vx++){
    if(SuperThreshold(iv->data[vx],valthresh,pksign)) nover ++;
    /*if(noabs){if(iv->data[vx] > valthresh) nover ++;}
      else{if(fabs(iv->data[vx]) > fabs(valthresh)) nover ++;}*/
  }

  clist = AllocClusterList(nover);
  clist->valthresh = valthresh;
  clist->pksign    = pksign;

  nover = 0;
  vx = 0;
  for(s=0;s<iv->nslices;s++){
    for(r=0;r<iv->nrows;r++){
      for(c=0;c<iv->ncols;c++){

	/*ok = 0;
	if(noabs){if(iv->data[vx]  > valthresh) ok = 1;}
	else{if(fabs(iv->data[vx]) > fabs(valthresh)) ok = 1;}*/

	if(SuperThreshold(iv->data[vx],valthresh,pksign)){
	  ss.s = s; ss.r = r; ss.c = c;
	  AddToCluster(&ss,clist->cluster[nover]);
	  nover ++;
	}
	vx++;
      }
    }
  }

  return(clist);
}
#endif
/* ------------------------------------------------------------- */
int CompareSrtStruct(const void *s1, const void *s2) /* for qsort */
{
  float v1, v2;
  v1 = ((SRTSTRUCT *)s1)->val;
  v2 = ((SRTSTRUCT *)s2)->val;

  if      ( v1 > v2) return(-1);
  else if ( v1 < v2) return(+1);
  return(0);
}

/* ------------------------------------------------------------- */
CLUSTER_LIST *SortClusterList(CLIST *clist, IMGVOL *iv)
{
  CLIST *clistsort;
  int n, s, r, c, i;
  SRTSTRUCT *srt;

  /* load up the sort structure */
  srt = (SRTSTRUCT *) calloc(sizeof(SRTSTRUCT),clist->nclusters);
  for(n=0;n<clist->nclusters;n++){
    i = clist->cluster[n]->ipeak;
    s = clist->cluster[n]->ss[i].s;
    r = clist->cluster[n]->ss[i].r;
    c = clist->cluster[n]->ss[i].c;
    srt[n].id  = n;
    if(clist->pksign > 0)      srt[n].val =  iv->vox[s][r][c];
    else if(clist->pksign < 0) srt[n].val = -iv->vox[s][r][c];
    else                       srt[n].val = fabs(iv->vox[s][r][c]);
  }

  /* sort */
  qsort(srt,clist->nclusters,sizeof(SRTSTRUCT),&CompareSrtStruct);

  clistsort = CopyClusterListHeader(NULL,clist);
  for(n=0;n<clist->nclusters;n++){
    i = srt[n].id;
    AddToClusterList(clist->cluster[i], clistsort);
  }
  clistsort->sorted = 1;
  return(clistsort);
}
/* ------------------------------------------------------ */
int FreeClusterList(void **ppclist)
{
  CLIST *clist;
  int n;

  clist = (CLIST *) *ppclist;

  if(clist->nclusters > 0){
    for(n=0; n < clist->nclusters; n++){
      if(clist->cluster[n])
	FreeCluster((void **)&(clist->cluster[n]));
    }
  }
  free(clist);
  *ppclist = NULL;
  return(0);
}
/* ------------------------------------------------------ */
int DumpClusterList(FILE *fp, void *pclist, 
		    IMGVOL *iv, char const *fmt)
{
  CLIST *clist;
  int n;

  clist = (CLIST *) pclist;
  fprintf(fp,"nclusters     = %d\n",clist->nclusters);
  fprintf(fp,"valthreshold  = %g\n",clist->valthresh);
  fprintf(fp,"distthreshold = %g\n",clist->distthresh);
  fprintf(fp,"sizethreshold = %d\n",clist->sizethresh);
  fprintf(fp,"pksign        = %d\n",clist->pksign);
  /*fprintf(fp,"noabs         = %d\n",clist->noabs);*/
  fprintf(fp,"sorted        = %d\n",clist->sorted);
  for(n=0; n < clist->nclusters; n++){
    fprintf(fp,"  ---Cluster %d ----\n",n);
    DumpCluster(fp,clist->cluster[n],iv,fmt);
  }

  return(0);
}
/* ------------------------------------------------------ */
int AddToClusterList(CLUSTER *cl, CLUSTER_LIST *clist)
{
  int n;

  n = clist->nclusters;
  clist->nclusters++;
  clist->cluster = (CLUSTER **) 
    realloc(clist->cluster,sizeof(CLUSTER *)* clist->nclusters);
  clist->cluster[n] = CopyCluster(NULL,cl);

  return(0);
}
/* ------------------------------------------------------------------------- */
int GrowClusterList(CLIST *clist, IMGVOL *iv, int nmax)
{
  int n;

  for(n=0; n < clist->nclusters; n++){
    GrowCluster(NULL,clist->cluster[n],iv,clist->valthresh,clist->pksign,nmax);
    /*FindClusterPeak(clist->cluster[n],iv,clist->pksign);*/
  }

  return(0);
}
/* ------------------------------------------------------------------------- */
int GrowCluster(VOLSS *seed, CLUSTER *cl, IMGVOL *iv, 
		float valthresh, int pksign, int ncallmax)
{
  static int call = 0;
  int s,r,c,deallocseed;
  VOLSS ss;

  deallocseed = 0;
  if(!seed){
    seed = (VOLSS *) calloc(sizeof(VOLSS),1);
    memcpy(seed, &(cl->ss[0]), sizeof(VOLSS));
    deallocseed = 1;
  }

  call ++;
  if(call > iv->nvox){
    fprintf(stderr,"ERROR: GrowCluster()\n");
    fprintf(stderr,"Apparent infinite recursion\n");
    exit(1);
  }

  for(s = seed->s-1; s <= seed->s+1; s++){
    if(s < 0 || s >= iv->nslices) continue;
    for(r = seed->r-1; r <= seed->r+1; r++){
      if(r < 0 || r >= iv->nrows) continue;
      for(c = seed->c-1; c <= seed->c+1; c++){
	if(c < 0 || c >= iv->ncols) continue;
	if(s==seed->s && r==seed->r && c==seed->c) continue;
	if(!SuperThreshold(iv->vox[s][r][c],valthresh,pksign)) continue;
	ss.s = s; ss.r = r; ss.c = c;
	if(IsClusterMember(&ss,cl)) continue;
	/*printf("s = %d, r = %d, c = %d \n",s,r,c);*/
	AddToCluster(&ss,cl);
	if(call < ncallmax || ncallmax < 0)
	  GrowCluster(&ss,cl,iv,valthresh,pksign,ncallmax);
      }
    }
  }
  
  if(deallocseed) free(seed);

  call--;
  return(0);
}
/* ------------------------------------------------------------ */
int IsClusterMember(VOLSS *ss, CLUSTER *cl)
{
  int n;
  for(n=0; n < cl->nvxs; n++){
    if( ss->s == cl->ss[n].s && ss->r == cl->ss[n].r && ss->c == cl->ss[n].c )
      return(1);
  }
  return(0);
}

/* ------------------------------------------------------------ */
int SuperThreshold(float v, float valthresh, int sign)
{
  if(v > valthresh && sign > 0) return(1);
  if(v < valthresh && sign < 0) return(1);
  if(fabs(v) > fabs(valthresh) && sign == 0) return(1);
  if(fabs(v) == fabs(valthresh)) return(1);
  return(0);
}
/* ------------------------------------------------------------------------- */
int FindClusterPeak(CLUSTER *cl, IMGVOL *iv, int pksign)
{
  int i, r,c,s;
  float val, pkval;

  cl->ipeak = 0;
  for(i=0; i < cl->nvxs; i++){
    s = cl->ss[i].s;
    r = cl->ss[i].r;
    c = cl->ss[i].c;
    if(i==0)  pkval = iv->vox[s][r][c];
    else {
      val = iv->vox[s][r][c];
      if(SuperThreshold(val,pkval,pksign)){
	pkval = val;
	cl->ipeak = i;
      }
    }
  }
  return(0);
}
/* ------------------------------------------------------------------------- */
CLUSTER_LIST * PruneBySize(CLIST *clist, int sizethresh)
{
  CLIST *clist2;
  int n;

  clist2 = CopyClusterListHeader(NULL,clist);
  clist2->sizethresh  = sizethresh;

  for(n=0; n < clist->nclusters; n++){
    if(clist->cluster[n]->nvxs >= sizethresh)
      AddToClusterList(clist->cluster[n],clist2);
  }
  return(clist2);
}
/* ------------------------------------------------------------------------- */
CLUSTER_LIST * PruneByDist(CLIST *clist, float distthresh, IMGVOL *iv)
{
  CLIST *clist2;
  int n,m,ipk, addtolist;
  float dx, dy, dz, d2, dth2, v1, v2;
  VOLSS *ss1, *ss2;

  dth2 = distthresh*distthresh;
  /*printf("dth2 = %g\n",dth2);*/

  clist2 = CopyClusterListHeader(NULL,clist);
  clist2->distthresh  = distthresh;

  for(n=0; n < clist->nclusters; n++){
    ipk = clist->cluster[n]->ipeak;
    ss1 = &(clist->cluster[n]->ss[ipk]);
    v1 = iv->vox[ss1->s][ss1->r][ss1->c];
    addtolist = 1;

    /* go through each previously OKed peak */
    for(m=0; m < clist2->nclusters; m++){
      ipk = clist2->cluster[m]->ipeak;
      ss2 = &(clist2->cluster[m]->ss[ipk]);
      dx = iv->sliceres * (ss1->s - ss2->s);
      dy = iv->rowres   * (ss1->r - ss2->r);
      dz = iv->colres   * (ss1->c - ss2->c);
      d2 = dx*dx + dy*dy + dz*dz;

      v2 = iv->vox[ss2->s][ss2->r][ss2->c];
      if(d2 < dth2){
	if(fabs(v1) != fabs(v2)){
	  if(!SuperThreshold(v1,v2,clist->pksign))
	    addtolist = 0;
	}
	else{
	  /* in case of a tie, use the one with the lowest index */
          /* note: talcoor uses the highest*/
	  addtolist = 0;
	}
      }

      /*printf("n=%d, m=%d, d2 = %5.0f, v1=%g, v2=%g, add=%d\n",
	n,m,d2,v1,v2,addtolist);*/
      if(addtolist == 0){
	break;
      }
    }/* end for m */

    if(addtolist){
      /*printf("Adding cluster %d to list\n",n);*/
      AddToClusterList(clist->cluster[n],clist2);
    }
    /* else printf("Excluding cluster %d from list\n",n);*/

  }

  return(clist2);
}
/* --------------------------------------------------------------- */
int PrintClusterReport(FILE *fp, CLIST *clist, IMGVOL *iv, 
		       char *infile, int no1988, char *CoorDef)
{
  int n, ipeak;
  char *sigch;
  VOLSS *ss;
  float tal_x, tal_y, tal_z, valpeak;

  if(clist->pksign > 0) sigch = "INCREASES";
  if(clist->pksign < 0) sigch = "DECREASES";
  if(clist->pksign == 0) sigch = "ABSOLUTE VALUE";

  fprintf(fp,"INPUT FILE: %s  (signal change %s)\n\n",infile,sigch);
  fprintf(fp,"Searching for peaks more significant than %g\n",fabs(clist->valthresh));
  fprintf(fp,"Peaks must be within a cluster of %d voxels.\n",clist->sizethresh);
  fprintf(fp,"Peaks must be separated by %g mm "
	  "or else the weighted location is kept.\n\n",clist->distthresh);

  fprintf(fp,"            X       Y       Z       -log(p)      s   r   c   n\n");
  fprintf(fp,"           ---     ---     ---      -------\n");
  for(n=0; n < clist->nclusters; n++){
    ipeak = clist->cluster[n]->ipeak;
    ss = &(clist->cluster[n]->ss[ipeak]);
    valpeak = iv->vox[ss->s][ss->r][ss->c];

#if 0
    tal_x = (ss->c - iv->col_x0) * iv->colres; /* convert to t88 coor */
    if(no1988)
      tal_y = (iv->row_y0 - ss->r) * iv->rowres;
    else
      tal_y = (iv->row_y0 - ss->r) * iv->rowres -12;/*correct by 12 for atlas*/
    tal_z = (iv->slice_z0 - ss->s) * iv->sliceres;		 
#endif
    ComputeTal(&tal_x,&tal_y,&tal_z,ss->c,ss->r,ss->s,
	       iv->col_x0,iv->row_y0,iv->slice_z0,
	       iv->colres,iv->rowres,iv->sliceres,
	       no1988,CoorDef);

    fprintf(fp,"pk%03d)    %4d    %4d    %4d      %8.4f    %3d %3d %3d %3d\n",
	    n+1,(int)tal_x,(int)tal_y,(int)tal_z,valpeak,ss->s,ss->r,ss->c,
	    clist->cluster[n]->nvxs);
  }

  return(0);
}
/* --------------------------------------------------------------- */
int ComputeTal(float *tal_x, float *tal_y, float *tal_z, 
	       int c, int r, int s, 
	       float col_x0, float row_y0, float slice_z0,
	       float colres, float rowres, float sliceres, 
	       int no1988, char *CoorDef)
{

  if(strcmp("RPI",CoorDef)==0){
    *tal_x = (c - col_x0) * colres; /* convert to t88 coor */
    if(no1988) *tal_y = (row_y0 - r) * rowres;
    else       *tal_y = (row_y0 - r) * rowres -12;/*correct by 12 for atlas*/
    *tal_z = (slice_z0 - s) * sliceres;		 
  }
  else if(strcmp("RIA",CoorDef)==0){
    *tal_x = (c - col_x0) * colres; /* convert to t88 coor */
    if(no1988) *tal_y = (s - row_y0) * rowres;
    else       *tal_y = (s - row_y0) * rowres - 12;
    *tal_z = (slice_z0 - r) * sliceres;
  }
  else {
    fprintf(stderr,"ERROR: unsupported coordef %s\n",CoorDef);
    exit (1);
  }

  return(0);
}

/* --------------------------------------------------------------- */
IMGVOL *LoadBVolumeAsImgVol(char *stemtemplate, char *ext, int frame)
{
  int slice,r,c,nslices, nrows, ncols, nframes, endian, ntmp, swap;
  char fname[2048];
  FILE *fp;
  IMGVOL *iv;
  float *tmp, *ptmp;

  if(!ext) ext = "bfloat";

  /* count number of slices */
  nslices = 0;
  sprintf(fname,stemtemplate,nslices);
  sprintf(fname,"%s.hdr",fname);
  fp = fopen(fname,"r");
  if(!fp){
    fprintf(stderr,"ERROR: LoadBVolumeAsImgVol()\n");
    fprintf(stderr,"Could not open %s\n",fname);
    exit(1);
  }
  while(fp){
    fclose(fp);
    nslices ++;
    sprintf(fname,stemtemplate,nslices);
    sprintf(fname,"%s.hdr",fname);
    fp = fopen(fname,"r");
  }

  /* get number of rows, cols, frames, and endianness */
  sprintf(fname,stemtemplate,0);
  sprintf(fname,"%s.hdr",fname);
  fp = fopen(fname,"r");
  fscanf(fp,"%d %d %d %d",&nrows,&ncols,&nframes,&endian);
  fclose(fp);

  if(frame >= nframes){
    fprintf(stderr,"ERROR: LoadBVolumeAsImgVol()\n");
    fprintf(stderr,"frame = %d >= nframes = %d\n",frame,nframes);
    exit(1);
  }

  iv = AllocImgVol(nslices,nrows,ncols);
  ntmp = nrows*ncols*nframes;
  tmp = (float *) calloc(sizeof(float), ntmp);
  for(slice=0; slice < nslices; slice++){
    sprintf(fname,stemtemplate,slice);
    sprintf(fname,"%s.%s",fname,ext);
    fp = fopen(fname,"r");
    if(!fp){
      fprintf(stderr,"ERROR: LoadBVolumeAsImgVol()\n");
      fprintf(stderr,"Could not open %s\n",fname);
      exit(1);
    }
    fread(tmp,sizeof(float),ntmp,fp);

    swap = 0;
#ifdef Linux
    if(endian==0) swap = 1;
#else
    if(endian==1) swap = 1;
#endif
    if(swap) byteswapbuf(tmp,ntmp*sizeof(float));

    ptmp = tmp + frame*nrows*ncols;
    for(r=0; r < nrows; r++){
      for(c=0; c < ncols; c++){
	iv->vox[slice][r][c] = *ptmp++;
      }
    }
  }

  free(tmp);
  return(iv);
}
/* ---------------------------------------------- */
int byteswapbuf(void *buf, long int nbufbytes)
{
  register char *cbuf,c;
  register long int n, nmax;

  nmax = nbufbytes;
  cbuf = (char *)buf;
  for(n=0;n<nmax;n+=4){
    c = *cbuf;
    *cbuf = *(cbuf+3);
    *(cbuf+3) = c;

    c = *(cbuf+1);
    *(cbuf+1) = *(cbuf+2);
    *(cbuf+2) = c;

    cbuf += 4;
  }
  return(0);
}
/* --------------------------------------------------------------- */
int SaveImgVolAsBStack(char *stem, IMGVOL *iv)
{
  char fname[2048];
  int  ntmp;
  float *tmp;
  FILE *fp;

  ntmp = iv->nslices * iv->nrows * iv->ncols;
  tmp  = (float *) calloc(sizeof(float),ntmp);
  memcpy(tmp,iv->data,sizeof(float)*ntmp);
#ifdef Linux
  byteswapbuf(tmp,ntmp*sizeof(float));
#endif

  sprintf(fname,"%s.hdr",stem);
  fp = fopen(fname,"w");
  fprintf(fp,"%3d %3d %2d 0\n",iv->nrows,iv->ncols,iv->nslices);
  fclose(fp);

  sprintf(fname,"%s.bfloat",stem);
  fp = fopen(fname,"w");
  fwrite(tmp,sizeof(float),ntmp,fp);
  fclose(fp);

  return(0);
}
/* --------------------------------------------------------------- */
IMGVOL *ClusterList2ImgVol(CLIST *clist, IMGVOL *ivsrc, int roidef)
{
  IMGVOL *iv;
  int n, ipeak;
  VOLSS *ss;
  
  iv = CopyImgVol(NULL,ivsrc);
  memset(iv->data,0,iv->nvox*sizeof(float));

  for(n=0; n < clist->nclusters; n++){
    ipeak = clist->cluster[n]->ipeak;
    ss = &(clist->cluster[n]->ss[ipeak]);
    if(roidef)
      iv->vox[ss->s][ss->r][ss->c] = n+1;
    else
      iv->vox[ss->s][ss->r][ss->c] = ivsrc->vox[ss->s][ss->r][ss->c];
   
  }

  return(iv);
}
